package connector;

import model.Driver;
import model.Sieve;
import view.Main;
import view.VNumber;
import view.VSieve;

public aspect Connector {
	VSieve lastSieve = null;
	Main canvas = null;

	int clickCount = 0;

	// to capture computation in the model
	pointcut createCanvas(Main canvas): 
		execution(void Main.setup()) && target(canvas);

	pointcut createSieve(Sieve s): execution(Sieve.new()) && this(s);

	pointcut newPrimeNumber(int prime) : 
		execution(void Sieve.testNotReady(int)) && args(prime);

	pointcut newTestNumber(int n): 
		call(void Sieve.test(int)) && args(n) && within(Driver);

	pointcut passSieve(Sieve dest, int n) : 
		call(void Sieve.test(int)) && target(dest) && args(n) &&
		withincode(void Sieve.testReady(int)) ;

	pointcut droppedBySieve(Sieve eater, int n) : 
		call(void Sieve.discard(int)) && args(n) && this(eater);

	// to capture user events
	pointcut mouseClicked(): 
		execution(void Main.mouseClicked());

	// to react computation in the model

	before(Main canvas): createCanvas(canvas){
		this.canvas = canvas;
	}

	after(Sieve s): createSieve(s) {
		// a new sieve will be shown
		if (lastSieve == null)
			lastSieve = new VSieve();
		else
			lastSieve = new VSieve(canvas, lastSieve);
		canvas.add(s, lastSieve);
	}

	before(int prime) :  newPrimeNumber(prime) {
		final VSieve l = lastSieve;
		// canvas.getVNumber(prime).moveTo(l);
		l.setPrime(prime);
		canvas.removeVNumber(prime);
	}

	void around(final int n) : newTestNumber(n) {
		canvas.add(new VNumber(canvas.getFirstVSieve(), n));
		waitMouseClick();
		new Thread() {
			public void run() {
				proceed(n);
			}
		}.start();
		try {
			Thread.sleep(500);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	private synchronized void waitMouseClick() {
		while (true) {
			if (clickCount > 0) {
				clickCount--;
				return;
			}
			try {
				wait();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	before(Sieve dest, int n):  passSieve(dest,n) {
		canvas.getVNumber(n).moveTo(canvas.getVSieve(dest));
	}

	before(Sieve eater, int n): droppedBySieve(eater,n){
		canvas.getVNumber(n).diveTo(canvas.getVSieve(eater));
		canvas.removeVNumber(n);
	}

	// to react user events
	after(): mouseClicked() {
		synchronized (this) {
			clickCount++;
			notifyAll();
		}
	}
}
